%% Axial Dispersion Model of a bioreactor 
% Solution Method: Orthogonol Collocation Method 

clear all
close all
clc
addpath('MWR_Library')
constant
% global p_out Tin v0_l v0_g rho_l MMASS mu_l g Ncomp FRAC_Lin...
%      FRAC_Gin R d_b d_c DELTAHrx H lambda_L D_l h_out t_w k_steel...
%      Nz 
global Nz D_l

tic 
Nz = 36;
zMIN = 0;      % Start of the reactor tube
zMAX = 12;      % End of the reactor tube

z = GLL_(Nz, zMIN, zMAX);   % Gauss-Lobatto-Lagrange 

derz  = LagrangeDerivativeMatrix_GLL(Nz);  % Derivative wrt z
derz  = derz * 2/(zMAX-zMIN);         % mapping from (-1,1) -> (zMIN, zMAX)
der2z = derz*derz;                    % Second derivative wrt z
Lagz    = eye(Nz); 


v_l     = v0_l;
T_wall  = ones(Nz,1)*312;            

% Liquid
wCH4l_in = FRAC_Lin(1);
wO2l_in  = FRAC_Lin(2);
wNH3l_in = FRAC_Lin(3);
wCO2l_in = FRAC_Lin(4);
wBIOl_in = FRAC_Lin(5);
wH2Ol_in = FRAC_Lin(6);

% Gas
wCH4g_in = FRAC_Gin(1);
wO2g_in  = FRAC_Gin(2);
wNH3g_in = FRAC_Gin(3);
wCO2g_in = FRAC_Gin(4);
wBIOg_in = FRAC_Gin(5);
wH2Og_in = FRAC_Gin(6);


wl_in    = [wCH4l_in; wO2l_in; wNH3l_in; wCO2l_in;...
            wBIOl_in; wH2Ol_in]';
wg_in    = [wCH4g_in; wO2g_in; wNH3g_in; wCO2g_in;...
            wBIOg_in; wH2Og_in]'; 
         
% Dispersion coefficient [m^2/s]
Re     = rho_l*v_l*d_c/mu_l;  
f      = 0.3164/Re^(1/4); 
Deff_l = 5.05*d_c*v_l*(f/2)^(1/2);  
Deff_g = Deff_l * (1-(d_b^(5/2)/(d_b^(5/2)+d_b^(5/2))));
  

resTol  = 1e-10;
relTol  = 1e-12;
alphawl = 1e-3;
alphawg = 1e-3; 
alphaT  = 2e-2;
alphaP  = 2e-2;
alphavg = 2e-2;

res_wl  = 1;
res_wg  = 1;
resP    = 1;
resvg   = 1;
resTl   = 1;
resTg   = 1;
rel_wl  = 1;
rel_wg  = 1;
relTl   = 1;
relTg   = 1;
relP    = 1;
relvg   = 1;

wg      = ones(Nz,1)*wg_in;
wl      = ones(Nz,1)*wl_in;
wBIO    = wl(:,5);
wCH4l   = wl(:,1);
wO2l    = wl(:,2);
wl_new  = zeros(Nz, Ncomp);
wg_new  = zeros(Nz, Ncomp);
P       = ones(Nz,1)*p_out;
v_g     = ones(Nz,1)*v0_g;
Tl      = Tin*ones(Nz,1);
Tg      = Tin*ones(Nz,1);
A       = zeros(Nz, Nz);
F       = zeros(Nz, 1);

%% Outer loop
iter = 0;
while res_wl > resTol || rel_wl > relTol || resTl > resTol... 
      || relTl > relTol || res_wg > resTol || rel_wg > relTol... 
      || resTg > resTol || relTg > relTol... 
      || resP > resTol  || relP > relTol...
      || resvg > resTol || relvg > relTol
    
    iter = iter + 1;       
    
    rel_wgi = 1;
    rel_wli = 1;
    relPi   = 1;
    relvgi  = 1;
    relTli  = 1;
    relTgi  = 1;
    
    P_old = P;
    v_old = v_g;
    wl_old = wl;
    wg_old = wg;
    Tl_old = Tl;
    Tg_old = Tg;
    %% --------------------Pressure-------------------------
    % Inner loop pressure
    iterp = 0;
    
    while relPi > 1e-12
     
        iterp = iterp + 1;
        
        % Hold-up [-]
        e_g = (v_g./(v_l+v_g));
        e_l = 1 - e_g;
        
        for i = 1:Nz
            F(i) = -e_l(i)*rho_l*g;
            for j = 1:Nz
                A(i,j) = derz(i,j);
            end
        end

        i = Nz;
        A(i,:) = 0;
        A(i,i) = Lagz(i,i); 
        F(i)   = p_out;

        P_new = A\F;

        AP    = A;
        FP    = F;
        P     = alphaP*P_new + (1-alphaP)*P;   % Under-relaxation
        relPi  = max(abs(P_new-P))/p_out;      % Relative tolerance

    end % End while P

    resP = ((AP*P-FP)'*(AP*P-FP))^(1/2)/p_out; % Residual

    %% ---------------------Gas Velocity--------------------    
    % Inner loop velocity
    iterv = 0;
    
    while relvgi > 1e-10
     
        iterv = iterv + 1;
        
        % Hold-up [-]
        e_g = (v_g./(v_l+v_g));
        e_l = 1 - e_g;
        
        % Contact area [m^2/m^3]
        a  = 6 * e_g/d_b;
        
        % Average Molar Mass

        M_mean = zeros(Nz,1); 
        for i = 1:Nz
            M_mean(i) = 1/(sum((wg(i,:)./MMASS)));
        end
        
        rho_G = zeros(Nz,1);
        for i = 1:Nz
            rho_G(i) = P(i)*M_mean(i)/(R*Tg(i));
        end
        

        % Derivative of the gas density
        dRHOg = zeros(Nz,1);
        for i = 1:Nz 
            dRHOg(i) = derz(i,:)*rho_G;
        end
        
        
        % Mass Transfer Coefficient
        kL =  masstransfercoeff(rho_G);
        

        gamma = zeros(Nz, Ncomp);        
        for i = 1:Nz
            for k = 1:Ncomp
                gamma(i, k) = -(kL(i,k)*a(i)*(wg(i,k)*P(i)*M_mean(i)/H(k)...
                    -rho_l*wl(i,k)));
            end
        end

        GAMMA = sum(gamma')';

        
        for i = 1:Nz
            F(i) = 0;
            for j = 1:Nz
                A(i,j) = rho_G(i)*derz(i,j)+ dRHOg(i)*Lagz(i,j) - GAMMA(i);
            end
        end              
        
        % Boundary conditions
        i = 1;
        A(i,:) = 0;
        A(i,i) = Lagz(i,i);
        F(i)   = v0_g;
                 
        vg_new = A\F;
        Avg = A;
        Fvg = F;
        v_g    = alphavg*vg_new + (1-alphavg)*v_g; % Under-relaxation 
        relvgi  = max(abs(vg_new-v_g))/v0_g;       % Relative tolerance
    end % End while v_g
 
    resvg = ((Avg*v_g-Fvg)'*(Avg*v_g-Fvg))^(1/2);  % Residual 
    %% ---------------------Mass Fractions-------------------
    
    % Inner loop mass fractions
    iterw  = 0;
    
    while rel_wli > 1e-10 || rel_wgi > 1e-10
        
        iterw = iterw  + 1;       
        
        % Hold-up [-]
        e_g = v_g./(v_l+v_g);
        e_l = 1 - e_g;
        
        % Contact area [m^2/m^3]
        a  = 6 * e_g/d_b;
        
        % Average Molar Mass
        M_mean = zeros(Nz,1); 
        for i = 1:Nz
            M_mean(i) = 1/(sum((wg(i,:)./MMASS)));
        end      
        
        % Reaction
        Rx = reactionrate(rho_l, wCH4l, wO2l, wBIO);       

        rho_G = zeros(Nz,1);
        for i = 1:Nz
            rho_G(i) = P(i)*M_mean(i)/(R*Tg(i));
        end     
        
        % Mass Transfer Coefficient
        kL = masstransfercoeff(rho_G);     
        
        gamma = zeros(Nz, Ncomp);        
        for i = 1:Nz
            for k = 1:Ncomp
                gamma(i, k) = -(kL(i,k)*a(i)*(wg(i,k)*P(i)*M_mean(i)/H(k)...
                    -rho_l*wl(i,k)));
            end
        end
    
        GAMMA = sum(gamma')';
        
        % Derivative of the gas density
        dRHOg = ones(Nz,1);
        for i = 1:Nz 
            dRHOg(i) = derz(i,:)*rho_G;
        end

        % Inlet gas density
        M_mean_in = 1/(sum(wg_in./MMASS));
        rho_Gin   = P(1) * M_mean_in/(R * Tin);
               
        % Derivative of the hold-up
        de_g = ones(Nz,1);
        for i = 1:Nz
            de_g(i) = derz(i,:)*e_g;
        end
        de_l = ones(Nz,1);
        for i = 1:Nz
            de_l(i) = derz(i,:)*e_l;
        end
        
        % Derivative of the gas velocity
        dv_g = ones(Nz,1);
        for i = 1:Nz
            dv_g(i) = derz(i,:)*v_g;
        end

                       
    % -------------------------Liquid----------------------- 
        for k = 1:Ncomp-1                   % For-loop components
            for i = 1:Nz                    % For-loop collocation points  
               F(i) = kL(i,k)*a(i)*wg(i,k)*P(i)*M_mean(i)/H(k)+...
                       e_l(i)*Rx(i,k);                  
                for j = 1:Nz
                    A(i,j) = rho_l * v_l * derz(i,j)-...
                     e_l(i) * rho_l * Deff_l * der2z(i,j)-...
                     Deff_l*rho_l*de_l(i)*derz(i,j)+...
                     kL(i,k)*a(i)*rho_l*Lagz(i,j);          
                end 
            end

            % Boundary conditions
            i = 1;
            A(i,:) = 0;
            A(i,i) = rho_l*v_l*Lagz(i,i);
            A(i,:) = A(i,:) - e_l(i)*rho_l*Deff_l*derz(i,:); 
            F(i)   = v0_l*rho_l*wl_in(k);
                     
            i = Nz;  
            A(i,:) = 0; 
            A(i,:) = derz(i,:);
            F(i) = 0; 
            
            wl_new(:, k) = A\F; 
            
            if k==3 % Store the A-matrix and F-vector of NH3
                ANH3 = A;
                FNH3 = F;
            end 
        end % End Ncomp-loop

        % w_H2O is calculated as 1-(sum(wl))
        wl_new(:,6) = ones(Nz,1) - sum(wl_new(:,1:5)')'; 
        
        % Under-relaxation
        wl = alphawl*wl_new + (1-alphawl)*wl;  
        
        % Relative tolerance H2O   
        rel_wli = max(abs(wl_new(:,6)-wl(:,6)));     
        wCH4l = wl(:,1);
        wO2l  = wl(:,2);
        wBIO  = wl(:,5);
 
    % -------------------------------- Gas --------------------------------            
    
       for k = [1,4] 
            for i = 1:Nz
               F(i) = rho_l*kL(i,k)*a(i)*wl(i,k);
                for j = 1:Nz
                A(i,j) = v_g(i)*rho_G(i)*derz(i,j)+...
                 GAMMA(i)*Lagz(i,j)-...    
                 e_g(i)*Deff_g*rho_G(i)*der2z(i,j)-...
                 e_g(i)*Deff_g*dRHOg(i)*derz(i,j)-... 
                 rho_G(i)*Deff_g*de_g(i)*derz(i,j)+...
                 kL(i,k)*a(i)*Lagz(i,j)*P(i)*M_mean(i)/H(k);
            end
        end
        
           
            % Boundary conditions
            i = 1;
            A(i,:) = 0;
            A(i,i) = v_g(i)*rho_G(i)*Lagz(i,i);
            A(i,:) = A(i,:) - e_g(i)*rho_G(i)*Deff_g*derz(i,:);
            F(i)   = v0_g*rho_Gin*wg_in(k);
                   
            i = Nz;
            A(i,:) = 0; 
            A(i,:) = derz(i,:);
            F(i)   = 0;
            
            wg_new(:, k) = A\F;
            
            if k==1             % Store the A-matrix and F-vector of CH4
                ACH4 = A;
                FCH4 = F;  
            end  
        end % End Ncomp-loop                                     
        
        wg_new(:,2) = ones(Nz,1) - sum(wg_new(:,[1,4])')'; % w_O2 is calculated as 1-(w_CH4+w_CO2)
        wg = alphawg*wg_new + (1-alphawg)*wg;    % Under-relaxation
        rel_wgi = max(abs(wg_new(:,2)-wg(:,2))); % Relative tolerance O2
        

    end % End while wl, wg
    res_wl = ((ANH3*wl(:,3)-FNH3)'*(ANH3*wl(:,3)-FNH3))^(1/2); % Residual liquid phase
    res_wg = ((ACH4*wg(:,1)-FCH4)'*(ACH4*wg(:,1)-FCH4))^(1/2); % Residual gas phase
    
    %% ---------------------Temperature---------------------
    
    % Inner loop temperature
    iterT  = 0;
    
    while relTli >1e-10 || relTgi > 1e-10
        
        iterT = iterT  + 1;
        
        % Hold-up [-]
        e_g = (v_g./(v_l+v_g));
        e_l = 1 - e_g;
        
        % Contact area [m^2/m^3]
        a  = 6 * e_g/d_b;

        % Inlet gas density
        M_mean_in = 1/(sum(wg_in./MMASS));
        rho_Gin   = P(1) * M_mean_in/(R * Tin); 
        
        % Average Molar Mass
        M_mean = zeros(Nz,1); 
        for i = 1:Nz
            M_mean(i) = 1/(sum((wg(i,:)./MMASS)));
        end
        
        rho_G = zeros(Nz,1);
        for i = 1:Nz
            rho_G(i) = P(i)*M_mean(i)/(R*Tg(i));
        end
         
        [cp_l, cp_g]     = heatcapacity(Tg, Tl, wl, wg);
        [cp_lin, cp_gin] = heatcapacity_inlet(wg_in, wl_in);

        
        % Reaction
        Rx = reactionrate(rho_l, wCH4l, wO2l, wBIO);       
        
        % Effective Thermal Conductivity [W/m*K]
        keff_l  = Deff_l * rho_l * cp_l;  
        keff_g  = Deff_g * rho_G .* cp_g;
        Dl_mean = sum((ones(Nz,1)*D_l.*wg(:,[1,2,4]))')';              
        
        % Mass Transfer Coefficient
        kL = masstransfercoeff(rho_G);

        % Mean mass transfer coefficient
        kL_mean = sum((kL.*wl)')';
        
        % Heat Transfer Coefficient [W/(m^2*K)]
        hL = zeros(Nz,1);
        for i = 1:Nz
            hL(i) = rho_l*kL_mean(i)*cp_l(i)*(mu_l/(rho_l*Dl_mean(i)))^(2/3)...
                  *(mu_l*cp_l(i)/lambda_L)^(-2/3);
        end
        
        hL_wall = zeros(Nz,1);
        for i = 1:Nz
            hL_wall(i) = 0.1*(rho_l^(0.75)*lambda_L^(0.5)*cp_l(i)^(0.5)...
                 *mu_l^(-0.25)*g^(0.25)*v_g(i)^(0.25));
        end

        
        %Overall Heat Transfer Coefficient
        U = zeros(Nz,1);
        for i = 1:Nz
            U(i) = 1/(1/hL_wall(i) + t_w/k_steel + 1/h_out);
        end
        
        % Derivative of the gas density
        dRHOg = ones(Nz,1);
        for i = 1:Nz 
            dRHOg(i) = derz(i,:)*rho_G;
        end
        
        %Derivative of the heat capacity
        dcpg = zeros(Nz,1);
        for i = 1:Nz
            dcpg(i) = derz(i,:)*cp_g;
        end
        dcpl = zeros(Nz,1);
        for i = 1:Nz
            dcpl(i) = derz(i,:)*cp_l;
        end
        
        %Derivative of the hold-up
        de_g = zeros(Nz,1);
        for i = 1:Nz
            de_g(i) = derz(i,:)*e_g;
        end
        de_l = zeros(Nz,1);
        for i = 1:Nz
            de_l(i) = derz(i,:)*e_l;
        end
        
        dTg = zeros(Nz,1);
        for i = 1:Nz
            dTg(i) = derz(i,:)*Tg;
        end
         
     % ------------------------Liquid-------------------------
        for i = 1:Nz           
            F(i) = e_l(i)*DELTAHrx*Rx(i,2)/(rho_l * v0_l * cp_lin)+...
                   hL(i)*a(i)*Tg(i)/(rho_l * v0_l * cp_lin)-...
                   4*U(i)*(Tl(i)-T_wall(i))/(d_c*rho_l * v0_l * cp_lin);                   
                for j=1:Nz
                    A(i,j) = rho_l * cp_l(i) * v_l * derz(i,j)/(rho_l * v0_l * cp_lin)-...
                             e_l(i)*Deff_l*rho_l*cp_l(i)*der2z(i,j)/(rho_l * v0_l * cp_lin)-...
                             e_l(i)*rho_l*Deff_l*dcpl(i)*derz(i,j)/(rho_l * v0_l * cp_lin)-...
                             rho_l*cp_l(i)*Deff_l*de_l(i)*derz(i,j)/(rho_l * v0_l * cp_lin)+...
                             hL(i)*a(i)*Lagz(i,j)/(rho_l * v0_l * cp_lin);
                end
        end


   
        % Boundary conditions
        i = 1;
        A(i,:) = 0;
        A(i,i) = rho_l*v_l*cp_l(i)*Lagz(i,i)/(rho_l * v0_l * cp_lin);
        A(i,:) = A(i,:) - e_l(i) * keff_l(i) * derz(i,:)/(rho_l * v0_l * cp_lin);
        F(i)   = rho_l * v0_l * cp_lin * Tin/(rho_l * v0_l * cp_lin);
     
        i = Nz;
        A(i,:) = 0; 
        A(i,:) = derz(i,:);
        F(i)   = 0;
        
        % Solving the equation system to find new T-vector
        Tl_new = A\F;
        ATl    = A;
        FTl    = F;
        Tl     = alphaT*Tl_new + (1-alphaT)*Tl;
        relTli  = max(abs(Tl_new-Tl))/Tin; 
        
    % -------------------------Gas--------------------------         
        for i = 1:Nz
            F(i) = hL(i)*a(i)*Tl(i)/(v0_g*cp_gin*rho_Gin);
                for j=1:Nz
                A(i,j) = rho_G(i) * cp_g(i) * v_g(i) * derz(i,j)/(v0_g*cp_gin*rho_Gin)-...
                        e_g(i)*Deff_g*rho_G(i)*cp_g(i)*der2z(i,j)/(v0_g*cp_gin*rho_Gin)-...
                        e_g(i)*Deff_g*rho_G(i)*dcpg(i)*derz(i,j)/(v0_g*cp_gin*rho_Gin)-...
                        e_g(i)*Deff_g*cp_g(i)*dRHOg(i)*derz(i,j)/(v0_g*cp_gin*rho_Gin)-...
                        Deff_g*cp_g(i)*rho_G(i)*de_g(i)*derz(i,j)/(v0_g*cp_gin*rho_Gin)+...
                        hL(i)*a(i)*Lagz(i,j)/(v0_g*cp_gin*rho_Gin);
                end
        end       

        
        % Boundary conditions
        i = 1;
        A(i,:) = 0;       
        A(i,i) = rho_G(i)*v_g(i)*cp_g(i)*Lagz(i,i)/(v0_g*cp_gin*rho_Gin);
        F(i)   = v0_g*rho_Gin*cp_gin*Tin/(v0_g*cp_gin*rho_Gin) +...
                 e_g(i)*Deff_g*rho_G(i)*cp_g(i)*dTg(i)/(v0_g*cp_gin*rho_Gin);
        
             
        i = Nz;
        A(i,:) = 0; 
        A(i,:) = derz(i,:);
        F(i)   = 0;
        
        % Solving the equation system to find new T-vector
        Tg_new = A\F;     
        ATg    = A;
        FTg    = F;
        Tg     = alphaT*Tg_new + (1-alphaT)*Tg;
        relTgi  = max(abs(Tg_new-Tg))/Tin;           

     end % End while Tl, Tg     
     resTl = ((ATl*Tl-FTl)'*(ATl*Tl-FTl))^(1/2);
     resTg = ((ATg*Tg-FTg)'*(ATg*Tg-FTg))^(1/2);
     
     
     relP   = max(abs(P-P_old))/p_out; 
     relvg  = max(abs(v_g-v_old))/v0_g;
     rel_wl = max(abs(wl(:,6)-wl_old(:,6))); 
     rel_wg = max(abs(wg(:,2)-wg_old(:,2)));
     relTl  = max(abs(Tl-Tl_old))/Tin; 
     relTg  = max(abs(Tg-Tg_old))/Tin;      
     

     res_wl, rel_wl, res_wg, rel_wg, resP, relP,...
     resvg, relvg, resTl, relTl, resTg, relTg
 
 
end % End outer loop

toc
t = toc

%  %% Mass test
%     % Derivative of the gas density
%     dRHOg = ones(Nz,1);
%     for i = 1:Nz 
%         dRHOg(i) = derz(i,:)*rho_G;
%     end
%  
%     % Derivative of vg
%     dv_g = ones(Nz,1);
%     for i = 1:Nz
%         dv_g(i) = derz(i,:)*v_g;
%     end
%     
%     %Derivative of wg: 
%     dwgdz = zeros(Nz,1); 
%     for i = 1:Nz
%         for k = 1:Ncomp
%             dwgdz(:,k) = derz(i,:) * wg(:,k);
%         end
%     end
%     
%     gamma = zeros(Nz, Ncomp);        
%     for i = 1:Nz
%         for k = 1:Ncomp
%             gamma(i, k) = -(kL(i,k)*a(i)*(wg(i,k)*P(i)*M_mean(i)/H(k)...
%                 -rho_l*wl(i,k)));
%         end
%     end
% 
%     GAMMA = sum(gamma')';
% 
%     M_mean = zeros(Nz,1); 
%     for i = 1:Nz
%         M_mean(i) = 1/(sum((wg(i,:)./MMASS)));
%     end
%         
%     % Calculating convective term for all species 
%     totmass_gas = ones(Nz,Ncomp); 
%     for i = 1:Nz
%         for k = 1:Ncomp
%             totmass_gas(i,k) = v_g(i) * rho_G(i) * dwgdz(i,k) ...
%                                 + v_g(i) * dRHOg(i) * wg(i,k) ...
%                                 + rho_G(i)*dv_g(i) * wg(i,k);
%         end
%     end
% 
%     % Summing the species convective terms 
%     totmass_gas_SUM = sum(totmass_gas')';
% 
%     % Continuity equation in gas phase 
%     continuity_gas = rho_G.*dv_g + dRHOg.*v_g + GAMMA;
% 
%     % MASS TRANSFER term from the mass balances 
%     kla_massbalance = ones(Nz,Ncomp);
%     for i = 1:Nz
%         for k = 1:Ncomp
%     kla_massbalance(i,k) = rho_l * kL(i,k) * a(i) * wl(i,k) - (kL(i,k) * a(i) * wg(i,k) * P(i) * M_mean(i))/H(k);
%         end
%     end
% 
%     SUMkla_massbalance = sum(kla_massbalance')';
% 
%     % TESTEN ER: 
% 
%     % -totmass_gas_SUM - SUMkla_massbalance + continuity_gas = 0
%     % Forskjellen er 1.0e-13
% 
% 
% 
%     % Derivative of wl
%     dwldz = zeros(Nz,1); 
%     for i = 1:Nz
%         for k = 1:Ncomp
%             dwldz(:,k) = derz(i,:) * wl(:,k);
%         end
%     end
%     
%     gamma_l = zeros(Nz, Ncomp);        
%     for i = 1:Nz
%         for k = 1:Ncomp
%             gamma_l(i, k) = (kL(i,k)*a(i)*(((wg(i,k)/MMASS(k)/SUM(i))*P(i)*MMASS(k)/H(k))-...
%                          rho_l*wl(i,k)));
%         end
%     end
% 
%     GAMMA_l = ones(Nz,1);
%     for i = 1:Nz
%         GAMMA_l(i) = sum(gamma_l(i,:));
%     end
% 
%     kla_massbalance = ones(Nz,Ncomp);
%     for i = 1:Nz
%         for k = 1:Ncomp
%     kla_massbalance(i,k) = -rho_l * kL(i,k) * a(i) * wl(i,k) + (kL(i,k) * a(i) * wg(i,k) * P(i) * M_mean(i))/H(k);
%         end
%     end
%     SUMkla_massbalance = sum(kla_massbalance')';
% 
%     % Continuity equation in liq phase 
%     
%     continuity_liq = - GAMMA_l;
%     totmass_liq = rho_l * v_l .* dwldz;
%     totmass_liq_SUM = sum(totmass_liq')';
%     
%     % TESTEN ER: 
% 
%     % totmass_liq_SUM + SUMkla_massbalance + continuity_liq = 0
%     % Forskjellen er 1e-10
%% Mass fraction profiles 
 
wg_title = {'CH_4 (gas)', 'O_2 (gas)', 'CO_2 (gas)'};
wg_ylabel = {'\omega_{CH4}', '\omega_{O2}', '\omega_{CO2}'};


wg_plot = [wg(:,1), wg(:,2), wg(:,4)];
figure(1)
%title('Mass fraction in gas phase')
for k = 1:3
    subplot(2,2,k)
    plot(z,wg_plot(:,k))
    xlim([0 zMAX])
    %ylim([0 max(wg_plot(:,i))])
    title(wg_title(k))
    ylabel(wg_ylabel(k))
    xlabel('z [m]')

end 

print('massfrac_gas.png','-dpng','-r300');

wl_title = {'CH_4 (liquid)', 'O_2 (liquid)', 'NH_3 (liquid)', ...
    'CO_2 (liquid)', 'Bio (liquid)', 'H_2O (liquid)'};
wl_ylabel = {'\omega_{CH4}', '\omega_{O2}', '\omega_{NH3}',...
    '\omega_{CO2}', '\omega_{Bio}', '\omega_{H2O}'};

wl_plot = [wl(:,1), wl(:,2), wl(:,3), wl(:,4), wl(:,5)-FRAC_Lin(5), wl(:,6)];
figure(2)
%title('Mass fraction in liquid phase')
for k = 1:Ncomp
    subplot(4,2,k)
    plot(z,wl_plot(:,k))
    xlim([0 zMAX])
    %ylim([0 max(wl(:,i))])
    title(wl_title(k))
    ylabel(wl_ylabel(k))
    xlabel('z [m]')
end 
print('massfrac_liq.png','-dpng','-r300');


%% Mole fraction    

% molemass = zeros(Nz,1);
% for i = 1:Nz
%     SUM = 0;
%     for k = 1:Ncomp
%         SUM = SUM + wg(i,k)/MMASS(k);
%     end
%     molemass(i) = SUM;
% end

SUMlin = 1/sum(FRAC_Lin./MMASS); 

ylin = zeros(1,Ncomp);
for k = 1:Ncomp
    ylin(k) = FRAC_Lin(k)/MMASS(k)*SUMlin;
end

SUMg = zeros(Nz,1);
for i = 1:Nz
    SUMg(i) = 1/sum(wg(i,:)./MMASS);
end

yg = zeros(Nz, Ncomp);
for k = 1:Ncomp
    for i = 1:Nz
        yg(i,k) = wg(i,k)/MMASS(k)*SUMg(i);
    end
end

SUMl = zeros(Nz,1);
for i = 1:Nz
    SUMl(i) = 1/sum(wl(i,:)./MMASS);
end

yl = zeros(Nz, Ncomp);
for k = 1:Ncomp
    for i = 1:Nz
        yl(i,k) = wl(i,k)/MMASS(k)*SUMl(i);
    end
end

yg_title = {'CH_4 (gas)', 'O_2 (gas)', 'CO_2 (gas)'};
yg_ylabel = {'y_{CH4}', 'y_{O2}', 'y_{CO2}'};


yg_plot = [yg(:,1), yg(:,2), yg(:,4)];

figure(3)
%title('Mole fraction in gas phase')
for k = 1:3
    subplot(2,2,k)
    plot(z,yg_plot(:,k))
    xlim([0 zMAX])
    title(yg_title(k))
    ylabel(yg_ylabel(k))
    xlabel('z [m]')

end 

print('molefrac_gas.png','-dpng','-r300');


yl_title = {'CH_4 (liquid)', 'O_2 (liquid)', 'NH_3 (liquid)', ...
    'CO_2 (liquid)', 'Bio (liquid)', 'H_2O (liquid)'};
yl_ylabel = {'y_{CH4}', 'y_{O2}', 'y_{NH3}',...
    'y_{CO2}', 'y_{Bio}', 'y_{H2O}'};

yl_plot = [yl(:,1), yl(:,2), yl(:,3), yl(:,4), yl(:,5)-ylin(5), yl(:,6)];

figure(4)
%title('Mole fraction in liquid phase')
for k = 1:Ncomp
    subplot(4,2,k)
    plot(z,yl_plot(:,k))
    xlim([0 zMAX])
    title(yl_title(k))
    ylabel(yl_ylabel(k))
    xlabel('z [m]')
end 
print('molefrac_liq.png','-dpng','-r300');         

%% Temperature    
T = [Tl-273, Tg-273];

T_title = {'Temperature (liquid)', 'Temperature (gas)'};
T_ylabel = {'T_L', 'T_G'};

figure(6)
%title('Temperature')
for i = 1:2
    subplot(1,2,i)
    plot(z,T(:,i))
    title(T_title(i))
    ylabel(T_ylabel(i))
    xlabel('z [m]')
end 
print('Temp.png','-dpng','-r300');

%% PLOTTING MASS FRACTIONS AS DENSITIES 


rhoCH4g = wg(:,1).*rho_G; 
rhoO2g  = wg(:,2).*rho_G; 
rhoCO2g = wg(:,4).*rho_G; 

rhog_title = {'CH4 (gas)', 'O2 (gas)', 'CO2 (gas)'};
rhog_ylabel = {'\rho_{CH4}', '\rho_{O2}', '\rho_{CO2}'};

rhog_plot = [rhoCH4g, rhoO2g, rhoCO2g];
figure(7)
for i = 1:3
    subplot(2,2,i)
    plot(z,rhog_plot(:,i))
    %xlim([0 zMAX])
    %ylim([0 max(rhog_plot(:,i))])
    title(rhog_title(i))
    ylabel(rhog_ylabel(i))

end 

print('gas-density.png','-dpng','-r300');    

rhoCH4l = wl(:,1).*rho_l; 
rhoO2l  = wl(:,2).*rho_l; 
rhoNH3l = wl(:,3).*rho_l;
rhoCO2l = wl(:,4).*rho_l;
rhoBIOl = wl(:,5).*rho_l;
rhoH2Ol = wl(:,6).*rho_l;

rhol_title = {'CH4 (liq)', 'O2 (liq)', 'NH3 (liq)', 'CO2 (liq)',...
                'BIO (liq)', 'H2O (liq)'};
rhol_ylabel = {'\rho_{CH4}', '\rho_{O2}', '\rho_{NH3}',...
                '\rho_{CO2}', '\rho_{BIO}', '\rho_{H2O}'};

rhol_plot = [rhoCH4l, rhoO2l, rhoNH3l, rhoCO2l, rhoBIOl, rhoH2Ol];

figure(8)
for i = 1:Ncomp
    subplot(4,2,i)
    plot(z,rhol_plot(:,i))
    %xlim([0 zMAX])
    %ylim([0 max(rhog_plot(:,i))])
    title(rhol_title(i))
    ylabel(rhol_ylabel(i))

end 

%print('liq-density.png','-dpng','-r300');    
% 
% figure(9)
% subplot(121)
% plot(z, rhoCH4l, 'r', z, rhoO2l, 'b', z, rhoCO2l)
% legend('CH_4', 'O_2', 'CO_2') 
% ylabel('Concentration in liquid[kg/m^3]')
% xlabel('z')
% 
% figure(9)
% subplot(122)
% plot(z, rhoCH4g, 'r', z, rhoO2g, 'b', z, rhoCO2g)
% legend('CH_4', 'O_2', 'CO_2') 
% ylabel('Concentration in gas[kg/m^3]')
% xlabel('z')
% %print('liq-density_comp.png','-dpng','-r300'); 


%% Concentration
M_mean = zeros(Nz,1); 
for i = 1:Nz
    M_mean(i) = 1/(sum(wg(i,:)./MMASS));
end
CH4g  = yg(:,1).*rho_G./M_mean; 
O2g  = yg(:,2).*rho_G./M_mean; 
CO2g  = yg(:,4).*rho_G./M_mean;

C_plot  = [CH4g, O2g, CO2g];
rhog_title  = {'CH4 (gas)', 'O2 (gas)', 'CO2 (gas)'};
rhog_ylabel = {'C_{CH4} [mol/m^3]', 'C_{O2} [mol/m^3]', 'C_{CO2} [mol/m^3]'};
figure(11)
for i = 1:3
    subplot(2,2,i)
    plot(z,C_plot(:,i), 'b')
    title(rhog_title(i))
    ylabel(rhog_ylabel(i))
    xlabel ('z [m]')
end

print('conc_gas.png','-dpng','-r300');

%% Pressure and Gas Velocity
figure(10)
subplot(121)
title('Pressure')
plot(z,P)
axis([0 zMAX 3e5 3.7e5])
xlabel('z [m]')
ylabel('P [Pa]')

figure(10)
subplot(122)
title('Gas Velocity')
plot(z,v_g)
axis([0 zMAX 0.04 0.05])
xlabel('z [m]')
ylabel('v_g [Pa]')

print('pressure_velocity.png','-dpng','-r300');

save('w_bio_0.02.mat', 'wg', 'wl', 'P', 'v_g', 'Tl', 'Tg', 'z', 'rho_G', 'Rx')