%% Axial Dispersion Model of a bioreactor 
% Solution Method: Orthogonol Collocation Method 

clear all
close all
clc
addpath('MWR_Library')
constant
% global p_out T v_l rho_l MMASS mu_l g Ncomp FRAC_Lin...
%      FRAC_Gin R d_b d_c DELTAHrx H lambda_L D_l h_out t_w k_steel...
%      Nt w_0
global Nt mu_l rho_l g 

tic 
Nt   = 20;                     % Number of collocation points
tMIN = 0;                      % Start of the reactor tube
tMAX = [6.39 9.9 17];          % Max time for different systems
%tMAX = [6.3 9.8 17];             
% Gas mass fractions
wg_in   = [FRAC_Gin(1); FRAC_Gin(2)]'; 
         
% Inlet Bubble Volume
V_in      = (pi/6)*db_in^3;
x_in      = 0.1; % x guess

% Pre-define variables
w_sys   = zeros(Nt,St);
db_sys  = zeros(Nt,St);
wg_sys  = zeros(Nt,Ncomp);
V_sys   = zeros(Nt,St);
pg_sys  = zeros(Nt,St);
x_sys   = zeros(Nt,St);
CD_sys  = zeros(Nt,St);
kL_sys  = zeros(Nt,St);


for s = 1:St % Systems
   
    t     = GLL_(Nt, tMIN, tMAX(s));   % Gauss-Lobatto-Lagrange  
    
    dert  = LagrangeDerivativeMatrix_GLL(Nt);  % Derivative wrt t
    dert  = dert * 2/(tMAX(s)-tMIN);         % mapping from (-1,1) -> (tMIN, tMAX)
    der2t = dert*dert;                    % Second derivative wrt t
    Lagt  = eye(Nt);
       
    
    resTol  = 1e-10;
    resTolw = 1e-7;
    relTol  = 1e-12;
    alphawg = 1e-3; 
    alphaw  = 1e-2;
    alphax  = 1e-3;
    alphaV  = 1e-2;
    alphadb = 1e-3;

    res_w   = 1;
    res_wg  = 1;
    res_db  = 1;
    res_V   = 1;
    rel_w   = 1;
    rel_wg  = 1;
    rel_db  = 1;
    rel_V   = 1;

    wg      = ones(Nt,1)*wg_in;
    wg_new  = ones(Nt, 1);
    pl      = ones(Nt,1)*p_out;
    w       = ones(Nt,1)*w_in(s);   %terminalhastigheten
    V       = ones(Nt,1)*V_in;
    db      = ones(Nt,1)*db_in;
    A       = zeros(Nt,Nt);
    F       = zeros(Nt,1);
    x       = ones(Nt,1)*x_in;

    %% Outer loop
    iter = 0;
    while res_w > resTolw || rel_w > relTol || res_wg > resTol...
            || rel_wg > relTol || res_V > resTol...
            || rel_V > relTol  || res_db > resTol || rel_db > relTol


        iter = iter + 1;

        rel_wgi = 1;
        rel_wi  = 1;
        rel_Vi  = 1;
        rel_dbi = 1;
        rel_xi  = 1;

        w_old   = w;
        wg_old  = wg;
        V_old   = V;
        db_old  = db;

        % Average Molar Mass
        M_mean = ones(Nt,1);
        for i = 1:Nt
            M_mean(i) = 1/(sum(wg(i,:)./MMASS));
        end


        derM = zeros(Nt,1);
        for i = 1:Nt
            derM(i) = dert(i,:)*M_mean;
        end

    %% ------------------------ Change in height ----------------------
        while rel_xi > 1e-12   
            for i = 1:Nt
                F(i) = w(i);
                for j = 1:Nt
                    A(i,j) = dert(i,j);
                end
            end

            % Initial Condition
            i = 1;
            A(i,:) = 0;
            A(i,i) = Lagt(i,i);
            F(i)   = tMIN;

            x_new   = A\F;
            rel_xi  = max(abs(x_new)-x);      
            x       = alphax*x_new + (1-alphax)*x;
            x(1) = abs(x(1));
        end

    %------------------------------------------------------------------       
        % Liquid pressure
        pl = p_out + rho_l*g*(zMAX-x);  

        % Gas Pressure
%         pg = pl-4*sig./db;
        pg = pl;
        dpg = -rho_l*g*w;    % -rho_l*g*dxdt

        % Bubble Area
        Ab = (pi/4)*db.^2;

        % Gas Density
        rho_G = ones(Nt,1);
        for i = 1:Nt
            rho_G(i) = pg(i)*M_mean(i)/(R*T);
        end
        
        % Bubble Reynolds Number
        Re = rho_l*w.*db./mu_l;
        
        % Mass Transfer Coefficient
        if s == 1           
            kL = masstransfer_clean(Re, db);
        elseif s == 2
            kL = masstransfer_part(Re, db);
        else
            kL= masstransfer_cont(Re, db);
        end

        % Sum of the mass flux
        gamma = zeros(Nt,1);        
        for i = 1:Nt
            for k = 1:Ncomp
                gamma(i,k) = kL(i,k)*pg(i)*M_mean(i)*wg(i,k)/H(k);
            end
        end

        GAMMA = ones(Nt,1);
        for i = 1:Nt
            GAMMA(i) = sum(gamma(i,:));
        end

        delta_rho = rho_l - rho_G; 

        % Eotvos Number
        E0       = delta_rho.*g.*(db.^2)./0.076;


        % Drag Coefficient
        if s == 1
            CD = max(min((16./Re).*(1 + 0.15*...
                         Re.^(0.687)), 48./Re),...
                         (8/3)*E0./(E0+4));         
        elseif s == 2
            CD = max(min((24./Re).*(1 + 0.15*...
                        Re.^(0.687)),72./Re),...
                        (8/3)*E0./(E0+4));       
        else
            CD = max((24./Re).*(1 + 0.15*...
                        Re.^(0.687)),(8/3)*E0./(E0+4));        
        end

       

       %% ----------------------- Momentum Balance ----------------------
       iterw = 0;

       while rel_wi > 1e-10

           iterw = iterw + 1;

            for i = 1:Nt
                F(i) = -g + rho_l*g/rho_G(i);
                for j = 1:Nt
                    A(i,j) = dert(i,j) +...
                        (1/2)*CD(i)*rho_l*abs(w(i))*Lagt(i,j)*Ab(i)/(rho_G(i)*V(i));
                end
            end

            % Inital Condition 
            i = 1;
            A(i,:) = 0;
            A(i,i) = Lagt(i,i);
            F(i)   = w_0;       % w_0 = 0

            w_new = A\F;

            Aw = A;
            Fw = F;
            rel_wi  = max(abs(w_new-w))/w_in(s);      
            w       = alphaw*w_new + (1-alphaw)*w;  
            w(1) = abs(w(1));
       end
       res_w      = ((Aw*w-Fw)'*(Aw*w-Fw))^(1/2);


        %% --------------------- Bubble Diameter ----------------------
        iterd = 0;

        while rel_dbi > 1e-10

            iterd = iterd + 1;

            for i = 1:Nt
               F(i) = -GAMMA(i)/(2*rho_G(i));
                  for j = 1:Nt
                      A(i,j) = dert(i,j)+(pg(i)*derM(i)+...
                          M_mean(i)*dpg(i))*Lagt(i,j)/(3*R*T*rho_G(i));
                  end
            end
 
            
            % Inital Condition 
            i = 1;
            A(i,:) = 0;
            A(i,i) = Lagt(i,i);
            F(i)   = db_in;

            db_new = A\F;

            Ad = A;
            Fd = F;

           rel_dbi  = max(abs(db_new-db))/db_in;      
           db       = alphadb*db_new + (1-alphadb)*db;

        end
        res_db      = ((Ad*db-Fd)'*(Ad*db-Fd))^(1/2);  


        %% --------------------- Total Mass Balance -------------------
        iterV = 0;     

        while rel_Vi > 1e-12

            iterV = iterV + 1;               

            % Derivative of the volume
            for i = 1:Nt
                F(i) = -GAMMA(i)*Ab(i)/rho_G(i);
                for j = 1:Nt
                    A(i,j) = dert(i,j) +...
                        Lagt(i,j)*pg(i)*derM(i)/(rho_G(i)*R*T) + ...
                        Lagt(i,j)*M_mean(i)*dpg(i)/(rho_G(i)*R*T);
                end
            end

            % Inital Condition 
            i = 1;
            A(i,:) = 0;
            A(i,i) = Lagt(i,i);
            F(i)   = V_in;

            V_new  = A\F;
            AV     = A;
            FV     = F;

            rel_Vi = max(abs(V_new-V))/V_in;      % Relative tolerance
            V      = alphaV*V_new + (1-alphaV)*V;

        end
        res_V      = ((AV*V-FV)'*(AV*V-FV))^(1/2);


        %% ---------------------Specie Mass Balance-------------------

        % Inner loop mass fractions
        iterwg  = 0;

        while rel_wgi > 1e-12

            iterwg = iterwg  + 1;    
         
            for k = 1:Ncomp
                for i = 1:Nt
                    F(i) = wg(i,k)*GAMMA(i)*Ab(i);
                    for j = 1:Nt
                        A(i,j) = V(i)*rho_G(i)*dert(i,j) +...
                            Ab(i)*kL(i,k)*M_mean(i)*pg(i)*Lagt(i,j)/H(k);
                    end
                end
            


                % Initial Condition
                i = 1;
                A(i,:) = 0;
                A(i,i) = Lagt(i,i);
                F(i)   = wg_in(k);


                wg_new(:,k) = A\F;

                if k==1             % Store the A-matrix and F-vector of O2
                    AO2 = A;
                    FO2 = F;  
                end
                
            end % End Ncomp

            rel_wgi = max(abs(wg_new(:,1)-wg(:,1))); % Relative tolerance O2
            wg = alphawg*wg_new + (1-alphawg)*wg;    % Under-relaxation
        end % End while wgi
        res_wg = ((AO2*wg(:,1)-FO2)'*(AO2*wg(:,1)-FO2))^(1/2); % Residual gas phase

        rel_w   = max(abs(w-w_old))/w_in(s);
        rel_wg  = max(abs(wg(:,1)-wg_old(:,1)));
        rel_db  = max(abs(db-db_old))/db_in; 
        rel_V   = max(abs(V-V_old))/V_in;

        res_w, rel_w, res_wg, rel_wg, res_db, rel_db, res_V, rel_V   

     end % End outer loop 

       % Collecting datas for the systems
       w_sys(:,s)     = w
       db_sys(:,s)    = db
       V_sys(:,s)     = V
       pg_sys(:,s)    = pg
       x_sys(:,s)     = x
       rho_G_sys(:,s) = rho_G
       CD_sys(:,s)    = CD

       
       if s ==1
           wg_clean = wg
           kL_clean = kL
       elseif s ==2
           wg_part = wg
           kL_part = kL
       else
           wg_cont = wg
           kL_cont = kL
       end
       
          
       t
       pause      
end % End system
toc
t = toc


figure(10)
plot(x, kL(:,1), 'b', x, kL(:,2), 'm')
legend('k_{L,O_2}, part', 'k_{L,N_2}, part');
set(gca, 'FontName', 'Times', 'FontSize', 12)
ylabel('k_L [m/s]', 'fontsize',12)
xlabel('z [m]', 'fontsize',12)
xlim([0 2])
print('kL_Dl_1.png','-dpng','-r900');



pause
figure(1)
plot(x_sys(:,1),db_sys(:,1), 'r', x_sys(:,2),db_sys(:,2), 'g',...
    x_sys(:,3),db_sys(:,3), 'b')
title('Bubble Size')
legend('Clean', 'Partly', 'Contaminated');
set(gca, 'FontName', 'Times', 'FontSize', 12)
ylabel('d_b [m]', 'fontsize',12)
xlabel('z [m]', 'fontsize',12)
xlim([0 2])
print('bubble_size.png','-dpng','-r900');

figure(2)
plot(x_sys(:,1),w_sys(:,1), 'r', x_sys(:,2),w_sys(:,2), 'g',...
    x_sys(:,3),w_sys(:,3), 'b')
title('Gas Velocity')
legend('Clean', 'Partly', 'Contaminated');
set(gca, 'FontName', 'Times', 'FontSize', 12)
ylabel('w [m/s]', 'fontsize',12)
xlabel('z [m]', 'fontsize',12)
xlim([0 2])
print('velocity.png','-dpng','-r900');

figure(3)
plot(x_sys(:,1),wg_clean(:,1), 'r', x_sys(:,1),wg_clean(:,2), 'g')
title('Mass Fraction Clean')
legend('O_2', 'N_2');
set(gca, 'FontName', 'Times', 'FontSize', 12)
ylabel('w_{g,k} [-]', 'fontsize',12)
xlabel('z [m]', 'fontsize',12)
xlim([0 2])
print('massfrac_clean.png','-dpng','-r900');

figure(4)
plot(x_sys(:,2),wg_part(:,1), 'r', x_sys(:,2),wg_part(:,2), 'g')
title('Mass Fraction Partly Contaminated')
legend('O_2', 'N_2');
set(gca, 'FontName', 'Times', 'FontSize', 12)
ylabel('w_{g,k} [-]', 'fontsize',12)
xlabel('z [m]', 'fontsize',12)
xlim([0 2])
print('massfrac_part.png','-dpng','-r900');

figure(5)
plot(x_sys(:,3),wg_cont(:,1), 'r', x_sys(:,3),wg_cont(:,2), 'g')
title('Mass Fraction Contaminated')
legend('O_2', 'N_2');
set(gca, 'FontName', 'Times', 'FontSize', 12)
ylabel('w_{g,k} [-]', 'fontsize',12)
xlabel('z [m]', 'fontsize',12)
xlim([0 2])
print('massfrac_cont.png','-dpng','-r900');

% fig = figure(3);
% [ax, h1, h2] = plotyy(x_sys(:,1), wg_clean(:,1), x_sys(:,1), wg_clean(:,2), 'plot');
% % Use the axis handles to set the labels of the y axes
% set(get(ax(1), 'Ylabel'), 'String', 'w_g [-]', 'fontsize',12);
% set(get(ax(2), 'Ylabel'), 'String', 't [s]', 'fontsize',12);
% 
% xlim([0 2])

figure(6)
plot(x_sys(:,1),pg_sys(:,1), 'r', x_sys(:,2),pg_sys(:,2), 'g',...
    x_sys(:,3),pg_sys(:,3), 'b')
title('Gas Pressure')
legend('Clean', 'Partly', 'Contaminated');
set(gca, 'FontName', 'Times', 'FontSize', 12)
ylabel('p_g [Pa]', 'fontsize',12)
xlabel('z [m]', 'fontsize',12)
xlim([0 2])
print('gas_pressure.png','-dpng','-r900');

figure(7)
plot(x_sys(:,1),V_sys(:,1), 'r', x_sys(:,2),V_sys(:,2), 'g',...
    x_sys(:,3),V_sys(:,3), 'b')
title('Bubble Volume')
legend('Clean', 'Partly', 'Contaminated');
set(gca, 'FontName', 'Times', 'FontSize', 12)
ylabel('V [m^3]', 'fontsize',12)
xlabel('z [m]', 'fontsize',12)
xlim([0 2])
print('volume.png','-dpng','-r900');

figure(8)
plot(x_sys(:,1),rho_G_sys(:,1), 'r', x_sys(:,2),rho_G_sys(:,2), 'g',...
    x_sys(:,3),rho_G_sys(:,3), 'b')
title('Total Gas Density')
legend('Clean', 'Partly', 'Contaminated');
set(gca, 'FontName', 'Times', 'FontSize', 12)
ylabel('\rho_{G,tot} [kg/m^3]', 'fontsize',12)
xlabel('z [m]', 'fontsize',12)
xlim([0 2])
print('totgas_density.png','-dpng','-r900');

% Clean
rho_O2_clean = rho_G_sys(:,1).*wg_clean(:,1);
rho_N2_clean = rho_G_sys(:,1).*wg_clean(:,2);
% Partly
rho_O2_part = rho_G_sys(:,2).*wg_part(:,1);
rho_N2_part = rho_G_sys(:,2).*wg_part(:,2);
% Cont
rho_O2_cont = rho_G_sys(:,3).*wg_cont(:,1);
rho_N2_cont = rho_G_sys(:,3).*wg_cont(:,2);

figure(9)
plot(x_sys(:,1),rho_O2_clean, 'r', x_sys(:,1),rho_N2_clean, 'g',...
    x_sys(:,2),rho_O2_part, 'b', x_sys(:,2),rho_N2_part, '.-',...
    x_sys(:,3), rho_O2_cont,'--', x_sys(:,3),rho_N2_cont, '--')
title('Component Gas Density')
legend('O_2, Clean','N_2, Clean', 'O_2, Partly', 'N_2, Partly',...
    'O_2, Contaminated', 'N_2, Contaminated');
set(gca, 'FontName', 'Times', 'FontSize', 12)
ylabel('\rho_{G,k} [kg_k/m^3]', 'fontsize',12)
xlabel('z [m]', 'fontsize',12)
xlim([0 2])

print('gas_density_comp.png','-dpng','-r900');

figure(10)
plot(x_sys(:,1),kL_clean(:,1), 'b',...
    x_sys(:,1),kL_clean(:,2), 'm', x_sys(:,2),kL_part(:,1), 'r*',...
    x_sys(:,2),kL_part(:,2), 'b--',...
    x_sys(:,3),kL_cont(:,1), 'm--', x_sys(:,3),kL_cont(:,2), 'b*')
legend('k_{L,O_2}, Clean,','k_{L,N_2}, Clean',...
    'k_{L,O_2}, part', 'k_{L,N_2}, part,','k_{L,O_2}, Contaminated',...
    'k_{L,N_2}, Contaminated');
set(gca, 'FontName', 'Times', 'FontSize', 12)
ylabel('k_L [m/s]', 'fontsize',12)
xlabel('z [m]', 'fontsize',12)
xlim([0 2])
print('kL.png','-dpng','-r900');

% save('SB_O2_N2_test_KL.mat', 'wg','w', 'V',...
%     'pg', 'db', 'x', 'rho_G', 't', 'kL')


save('SB_O2_N2_test_KL.mat', 'wg_clean', 'wg_part', 'wg_cont', 'w_sys', 'V_sys',...
    'pg_sys', 'db_sys', 'x_sys', 'rho_G_sys', 't', 'kL_clean','kL_part','kL_cont')
