%% Constants

global p_out Tin v0_l v0_g rho_l MMASS mu_l g rho_s Ncomp FRAC_Lin...
    FRAC_Gin R d_b YIELD d_c DELTAHrx H lambda_L D_l h_out t_w...
    k_steel CP


% Initial Data
Tin    = 45+273;           % [K]
p_out  = 3e5;              % [Pa]
v0_l   = 0.4;              % [m/s]
v0_g   = 0.05;             % [m/s]
Ncomp  = 6;

% Initial Mass Fraction [kg/kg]
% Liquid
FRAC_Lin(1) = 0;                    % CH4
FRAC_Lin(2) = 0;                    % O2
FRAC_Lin(3) = 10/2050;              % NH3
FRAC_Lin(4) = 0;                    % CO2
FRAC_Lin(5) = 0.03;                 % Biomass
FRAC_Lin(6) = 1-sum(FRAC_Lin(1:5)); % H2O

% Gas
FRAC_Gin(1) = 0.23;                 % CH4
FRAC_Gin(2) = 1-FRAC_Gin(1);        % O2
FRAC_Gin(3) = 0;                    % NH3
FRAC_Gin(4) = 0;                    % CO2
FRAC_Gin(5) = 0;                    % Biomass
FRAC_Gin(6) = 0;                    % H2O

% Molemass of the Components [kg/mol]
MMASS(1)   = 16.04e-3;      % CH4
MMASS(2)   = 32.00e-3;      % O2
MMASS(3)   = 17.03e-3;      % NH3
MMASS(4)   = 44.01e-3;      % CO2
MMASS(5)   = 24.63e-3;      % Biomass CH(1.8)O(0.5)N(0.2)
MMASS(6)   = 18.02e-3;      % H2O


% At (45 degrees, 3 atm)
mu_l   = 5.96e-4;           % Viscosity water, [Pa*s]
rho_l  = 990.24;            % Density water, [kg/m^3]
rho_s  = 990.24;            % Density biomass [kg/m^3] 

% Yield [mol(component)/mol(biomass)]
YIELD(1) = -1/0.52;         % CH4
YIELD(2) = -1.454/0.52;     % O2
YIELD(3) = -0.104/0.52;     % NH3
YIELD(4) = 0.48/0.52;       % CO2
YIELD(5) = 1.00;            % Bio
YIELD(6) = 1.688/0.52;      % H2O


% Diameters
d_b = (2e-3);   % Diameter bubble [m]
d_c = 0.2;   % Diameter column [m]


g = 9.80665;    % Gravitational constant [m/s^2] 
R = 8.314459;   % Universal gas constant [m^3 Pa/mol*K]

DELTAHrx = -460e3/MMASS(2);  % Heat of reaction [J/kg]

% Henry's constant [m^3 Pa/mol]
H(1) = 1000.9e2;          % CH4
H(2) = 1100.81e2;         % O2
H(3) = 10e10;             % NH3
H(4) = 47.39e2;           % CO2
H(5) = 10e10;             % Bio
H(6) = 10e10;             % H2O


% Thermal conductivity slurry [W/mK]
lambda_L = 638e-3;
 
 % Diffusion coefficient [m^2/s]
D_l = [2.41e-9 3.05e-9 3.5e-9]; % Diffusion coefficient (CH4, O2 , CO2)
 
k_steel = 16;       % [W/(m*K)]
t_w     = 0.01;     % [m]
h_out   = 1000;     % [W/(m^2*K)]
 
% Heat capasity coefficients for the components 
% Source: NIST Chemistry WebBook http://webbook.nist.gov
%------------------------------------------------------------------------

CP(1,1) = -0.7030295;               % 1. coefficient for CH4   [J/moleK]
CP(1,2) = 108.4773;                 % 2. coefficient for CH4   [J/moleK^2]
CP(1,3) = -42.52157;                % 3. coefficient for CH4   [J/moleK^3]
CP(1,4) = 5.862788;                 % 4. coefficient for CH4   [J/moleK^4]
 
CP(2,1) = 31.32234;                 % 1. coefficient for O2   [J/moleK] 
CP(2,2) = -20.23531;                % 2. coefficient for O2   [J/moleK^2]
CP(2,3) = 57.86644;                 % 3. coefficient for O2   [J/moleK^3]
CP(2,4) = -36.50624;                % 4. coefficient for O2   [J/moleK^4]
 
CP(3,1) = 19.99563;                 % 1. coefficient for NH3   [J/moleK]
CP(3,2) = 49.77119;                 % 2. coefficient for NH3   [J/moleK^2]
CP(3,3) = -15.37599;                % 3. coefficient for NH3   [J/moleK^3]
CP(3,4) = 1.921168;                 % 4. coefficient for NH3   [J/moleK^4]
 
CP(4,1) = 24.99735;                 % 1. coefficient for CO2   [J/moleK]
CP(4,2) = 55.18696;                 % 2. coefficient for CO2   [J/moleK^2]
CP(4,3) = -33.69137;                % 3. coefficient for CO2   [J/moleK^3]
CP(4,4) = 7.948387;                 % 4. coefficient for CO2   [J/moleK^4]
 
% Assumed to be the same as the Cp of water 
CP(5,1) = -203.6060;                % 1. coefficient for X  [J/moleK]
CP(5,2) = 1523.290;                 % 2. coefficient for X  [J/moleK^2]
CP(5,3) = -3196.413;                % 3. coefficient for X  [J/moleK^3]
CP(5,4) = 2474.455;                 % 4. coefficient for X  [J/moleK^4]
 
CP(6,1) = -203.6060;                % 1. coefficient for H2O   [J/moleK]          
CP(6,2) = 1523.290;                 % 2. coefficient for H2O   [J/moleK^2]
CP(6,3) = -3196.413;                % 3. coefficient for H2O   [J/moleK^3]
CP(6,4) = 2474.455;                 % 4. coefficient for H2O   [J/moleK^4] 
 
 
